@@ -90,7 +90,7 @@ except (ImportError, ValueError):
     import utils
 
 
-PROTO_VERSION = '0.1'
+PROTO_VERSION = '0.11'
 CLIENT = 'flootty'
 INITIAL_RECONNECT_DELAY = 1000
 FD_READ_BYTES = 65536
@@ -98,6 +98,8 @@ FD_READ_BYTES = 65536
 SELECT_TIMEOUT = 0.1
 NET_TIMEOUT = 10
 MAX_BYTES_TO_BUFFER = 65536
+DEFAULT_HOST = "floobits.com"
+DEFAULT_PORT = 3448
 
 
 def read_floorc():
@@ -189,10 +191,12 @@ def parse_url(workspace_url):
         'secure': secure,
     }
 
+usage = '''usage: %prog [options] [terminal_name]\n
+For more help, see https://github.com/Floobits/flootty'''
+
 
 def main():
     settings = read_floorc()
-    usage = "usage: %prog  --workspace=WORKSPACE --owner=OWNER [options] term_name.\n\n\tSee https://github.com/Floobits/flootty"
     parser = optparse.OptionParser(usage=usage)
 
     parser.add_option("-u", "--username",
@@ -205,41 +209,41 @@ def main():
                       default=settings.get('secret'),
                       help="Your Floobits secret (api key)")
 
-    parser.add_option("--host",
-                      dest="host",
-                      default="floobits.com",
-                      help="The host to connect to")
-
-    parser.add_option("-p", "--port",
-                      dest="port",
-                      default=3448,
-                      help="The port to connect to")
-
     parser.add_option("-c", "--create",
                       dest="create",
                       default=False,
                       action="store_true",
                       help="The terminal name to create")
 
+    parser.add_option("--host",
+                      dest="host",
+                      default=DEFAULT_HOST,
+                      help="The host to connect to. Deprecated. Use --url instead.")
+
+    parser.add_option("-p", "--port",
+                      dest="port",
+                      default=DEFAULT_PORT,
+                      help="The port to connect to. Deprecated. Use --url instead.")
+
     parser.add_option("-w", "--workspace",
                       dest="workspace",
-                      help="The workspace name")
+                      help="The workspace name. --owner is required with this option. Deprecated. Use --url instead.")
 
     parser.add_option("-o", "--owner",
                       dest="owner",
-                      help="The workspace owner")
+                      help="The workspace owner. --workspace is required with this option. Deprecated. Use --url instead.")
 
     parser.add_option("-l", "--list",
                       dest="list",
                       default=False,
                       action="store_true",
-                      help="List all ptys in the workspace")
+                      help="List all terminals in the workspace")
 
     parser.add_option("--unsafe",
                       dest="safe",
                       default=True,
                       action="store_false",
-                      help="Less safe terminal. This allows other users to send enter.")
+                      help="Less safe terminal. This allows other users to send enter in your terminal.")
 
     parser.add_option("--no-ssl",
                       dest="use_ssl",
@@ -250,7 +254,13 @@ def main():
     parser.add_option("--url",
                       dest="workspace_url",
                       default=None,
-                      help="The URL of the workspace to connect to. This is a convenience for copy-pasting from the browser.")
+                      help="The URL of the workspace to connect to.")
+
+    parser.add_option("--resize",
+                      dest="resize",
+                      default=False,
+                      action="store_true",
+                      help="Resize your terminal to the host terminal size.")
 
     options, args = parser.parse_args()
 
@@ -264,8 +274,16 @@ def main():
     term_name = args and args[0] or default_term_name
 
     if options.workspace and options.owner and options.workspace_url:
+        # TODO: confusing
         parser.error("You can either specify --workspace and --owner, or --url, but not both.")
 
+    if bool(options.workspace) != bool(options.owner):
+        parser.error("You must specify a workspace and owner or neither.")
+
+    for opt in ['owner', 'workspace']:
+        if getattr(options, opt):
+            print('%s is deprecated. Please use --url instead.' % opt)
+
     if not options.workspace or not options.owner:
         floo = {}
         if options.workspace_url:
@@ -279,8 +297,10 @@ def main():
                     pass
                 else:
                     break
+        options.host = floo.get('host')
         options.workspace = floo.get('workspace')
         options.owner = floo.get('owner')
+        options.use_ssl = floo.get('secure')
         if not options.port:
             options.port = floo.get('port')
         if not options.host:
@@ -546,7 +566,12 @@ class Flootty(object):
         return self.join_term()
 
     def on_disconnect(self, data):
-        die('You were disconnected because: %s.' % data.get('reason', '').lower())
+        reason = data.get('reason')
+        out('Disconnected by server!')
+        if reason:
+            # TODO: don't kill terminal until current process is done or something
+            die('Reason: %s' % reason)
+        self.reconnect()
 
     def on_error(self, data):
         if self.term_id is None:
@@ -575,7 +600,7 @@ class Flootty(object):
             return
         if not self.options.create:
             return
-        self.handle_stdio(base64.b64decode(data['data']))
+        self.handle_stdio(base64.b64decode(data['data']), data.get('user_id'))
 
     def on_term_stdout(self, data):
         if data.get('id') != self.term_id:
@@ -642,7 +667,7 @@ class Flootty(object):
             self.sock = ssl.wrap_socket(self.sock, ca_certs=self.cert_fd.name, cert_reqs=ssl.CERT_REQUIRED)
         elif self.port == 3448:
             self.port = 3148
-        out('Connecting to %s:%s.' % (self.host, self.port))
+        out('Connecting to %s' % self.workspace_url())
         try:
             self.sock.connect((self.host, self.port))
             if self.options.use_ssl:
@@ -666,7 +691,7 @@ class Flootty(object):
         else:
             if self.port != 3148:
                 port_str = ':%s' % self.port
-        return '%s://%s%s/%s/%s/' % (proto_str, self.host, port_str, self.owner, self.workspace)
+        return '%s://%s%s/%s/%s' % (proto_str, self.host, port_str, self.owner, self.workspace)
 
     def join_term(self):
         out('Successfully joined %s' % (self.workspace_url()))
@@ -755,10 +780,23 @@ class Flootty(object):
 
         self.add_fd(pty.STDIN_FILENO, reader=stdin_write, name='create_term_stdin_write')
 
-        def net_stdin_write(buf):
+        def net_stdin_write(buf, user_id=None):
             if self.options.safe:
-                buf = buf.replace('\n', '')
-                buf = buf.replace('\r', '')
+                if buf.find('\n') != -1 or buf.find('\r') != -1:
+                    to = user_id or []
+                    self.transport('datamsg', {
+                        'to': to,
+                        'data': {
+                            'name': 'safe_term',
+                            'term_id': self.term_id,
+                            'msg': 'Terminal %s is in safe mode. Other users are not allowed to press enter.' % self.term_name,
+                        }})
+                    self.transport('term_stdout', {
+                        'id': self.term_id,
+                        'data': base64.b64encode('\a').decode('utf8'),
+                        })
+                    buf = buf.replace('\n', '')
+                    buf = buf.replace('\r', '')
                 if not buf:
                     return
             write(self.master_fd, buf)
@@ -804,7 +842,8 @@ class Flootty(object):
                 self.transport('update_term', {'id': self.term_id, 'size': [buf[1], buf[0]]})
         else:
             # XXXX: this resizes the window :/
-            #os.write(pty.STDOUT_FILENO, "\x1b[8;{rows};{cols}t".format(rows=buf[0], cols=buf[1]))
+            if self.options.resize:
+                os.write(pty.STDOUT_FILENO, "\x1b[8;{rows};{cols}t".format(rows=buf[0], cols=buf[1]))
             fcntl.ioctl(pty.STDOUT_FILENO, termios.TIOCSWINSZ, buf)
 
     def cleanup(self):
